@@ -50,7 +50,7 @@ __lws_reset_wsi(struct lws *wsi)
 	 * if we have wsi in our transaction queue, if we are closing we
 	 * must go through and close all those first
 	 */
-	if (wsi->vhost) {
+	if (wsi->a.vhost) {
 
 		/* we are no longer an active client connection that can piggyback */
 		lws_dll2_remove(&wsi->dll_cli_active_conns);
@@ -71,21 +71,39 @@ __lws_reset_wsi(struct lws *wsi)
 	}
 #endif
 
-	if (wsi->vhost)
+	if (wsi->a.vhost) {
+		lws_vhost_lock(wsi->a.vhost);
 		lws_dll2_remove(&wsi->vh_awaiting_socket);
+		lws_vhost_unlock(wsi->a.vhost);
+	}
 
 	/*
 	 * Protocol user data may be allocated either internally by lws
 	 * or by specified the user. We should only free what we allocated.
 	 */
-	if (wsi->protocol && wsi->protocol->per_session_data_size &&
-	    wsi->user_space && !wsi->user_space_externally_allocated)
+	if (wsi->a.protocol && wsi->a.protocol->per_session_data_size &&
+	    wsi->user_space && !wsi->user_space_externally_allocated) {
+		/* confirm no sul left scheduled in user data itself */
+		lws_sul_debug_zombies(wsi->a.context, wsi->user_space,
+				wsi->a.protocol->per_session_data_size, __func__);
 		lws_free_set_NULL(wsi->user_space);
+	}
+
+	/*
+	 * Don't let buflist content or state from the wsi's previous life
+	 * carry over to the new life
+	 */
 
 	lws_buflist_destroy_all_segments(&wsi->buflist);
+	lws_dll2_remove(&wsi->dll_buflist);
 	lws_buflist_destroy_all_segments(&wsi->buflist_out);
 #if defined(LWS_WITH_UDP)
-	lws_free_set_NULL(wsi->udp);
+	if (wsi->udp) {
+		/* confirm no sul left scheduled in wsi->udp itself */
+		lws_sul_debug_zombies(wsi->a.context, wsi->udp,
+				      sizeof(*wsi->udp), "close udp wsi");
+		lws_free_set_NULL(wsi->udp);
+	}
 #endif
 	wsi->retry = 0;
 
@@ -103,13 +121,13 @@ __lws_reset_wsi(struct lws *wsi)
 		lws_buflist_destroy_all_segments(&wsi->http.buflist_post_body);
 #endif
 
-	if (wsi->vhost && wsi->vhost->lserv_wsi == wsi)
-		wsi->vhost->lserv_wsi = NULL;
+	if (wsi->a.vhost && wsi->a.vhost->lserv_wsi == wsi)
+		wsi->a.vhost->lserv_wsi = NULL;
 #if defined(LWS_WITH_CLIENT)
-	if (wsi->vhost)
+	if (wsi->a.vhost)
 		lws_dll2_remove(&wsi->dll_cli_active_conns);
 #endif
-	wsi->context->count_wsi_allocated--;
+	wsi->a.context->count_wsi_allocated--;
 
 	__lws_same_vh_protocol_remove(wsi);
 #if defined(LWS_WITH_CLIENT)
@@ -118,7 +136,7 @@ __lws_reset_wsi(struct lws *wsi)
 #endif
 
 #if defined(LWS_WITH_PEER_LIMITS)
-	lws_peer_track_wsi_close(wsi->context, wsi->peer);
+	lws_peer_track_wsi_close(wsi->a.context, wsi->peer);
 	wsi->peer = NULL;
 #endif
 
@@ -144,15 +162,19 @@ __lws_free_wsi(struct lws *wsi)
 		return;
 
 	__lws_reset_wsi(wsi);
+	__lws_wsi_remove_from_sul(wsi);
 
-	if (wsi->context->event_loop_ops->destroy_wsi)
-		wsi->context->event_loop_ops->destroy_wsi(wsi);
+	if (wsi->a.context->event_loop_ops->destroy_wsi)
+		wsi->a.context->event_loop_ops->destroy_wsi(wsi);
 
 	lws_vhost_unbind_wsi(wsi);
 
 	lwsl_debug("%s: %p, remaining wsi %d, tsi fds count %d\n", __func__, wsi,
-			wsi->context->count_wsi_allocated,
-			wsi->context->pt[(int)wsi->tsi].fds_count);
+			wsi->a.context->count_wsi_allocated,
+			wsi->a.context->pt[(int)wsi->tsi].fds_count);
+
+	/* confirm no sul left scheduled in wsi itself */
+	lws_sul_debug_zombies(wsi->a.context, wsi, sizeof(wsi), __func__);
 
 	lws_free(wsi);
 }
@@ -174,8 +196,8 @@ lws_remove_child_from_any_parent(struct lws *wsi)
 			lwsl_info("%s: detach %p from parent %p\n", __func__,
 				  wsi, wsi->parent);
 
-			if (wsi->parent->protocol)
-				wsi->parent->protocol->callback(wsi,
+			if (wsi->parent->a.protocol)
+				wsi->parent->a.protocol->callback(wsi,
 						LWS_CALLBACK_CHILD_CLOSING,
 					       wsi->parent->user_space, wsi, 0);
 
@@ -201,14 +223,14 @@ lws_inform_client_conn_fail(struct lws *wsi, void *arg, size_t len)
 		return;
 
 	wsi->already_did_cce = 1;
-	lws_stats_bump(&wsi->context->pt[(int)wsi->tsi],
+	lws_stats_bump(&wsi->a.context->pt[(int)wsi->tsi],
 		       LWSSTATS_C_CONNS_CLIENT_FAILED, 1);
 
-	if (!wsi->protocol)
+	if (!wsi->a.protocol)
 		return;
 
 	if (!wsi->client_suppress_CONNECTION_ERROR)
-		wsi->protocol->callback(wsi,
+		wsi->a.protocol->callback(wsi,
 					LWS_CALLBACK_CLIENT_CONNECTION_ERROR,
 					wsi->user_space, arg, len);
 }
@@ -251,8 +273,11 @@ __lws_close_free_wsi(struct lws *wsi, enum lws_close_status reason,
 		lwsl_info("%s: wsi %p: going down with stuff in buflist\n",
 				__func__, wsi); }
 
-	context = wsi->context;
+	context = wsi->a.context;
 	pt = &context->pt[(int)wsi->tsi];
+
+	lws_pt_assert_lock_held(pt);
+
 	lws_stats_bump(pt, LWSSTATS_C_API_CLOSE, 1);
 
 #if defined(LWS_WITH_CLIENT)
@@ -272,7 +297,7 @@ __lws_close_free_wsi(struct lws *wsi, enum lws_close_status reason,
 		wsi2 = wsi->child_list;
 		while (wsi2) {
 			wsi1 = wsi2->sibling_list;
-			wsi2->parent = NULL;
+//			wsi2->parent = NULL;
 			/* stop it doing shutdown processing */
 			wsi2->socket_is_permanently_unusable = 1;
 			__lws_close_free_wsi(wsi2, reason,
@@ -286,8 +311,8 @@ __lws_close_free_wsi(struct lws *wsi, enum lws_close_status reason,
 	if (wsi->role_ops == &role_ops_raw_file) {
 		lws_remove_child_from_any_parent(wsi);
 		__remove_wsi_socket_from_fds(wsi);
-		if (wsi->protocol)
-			wsi->protocol->callback(wsi, wsi->role_ops->close_cb[0],
+		if (wsi->a.protocol)
+			wsi->a.protocol->callback(wsi, wsi->role_ops->close_cb[0],
 					wsi->user_space, NULL, 0);
 		goto async_close;
 	}
@@ -303,11 +328,16 @@ __lws_close_free_wsi(struct lws *wsi, enum lws_close_status reason,
 		/* we are not a network connection, but a handler for CGI io */
 		if (wsi->parent && wsi->parent->http.cgi) {
 
-			if (wsi->parent->child_list == wsi && !wsi->sibling_list)
-				lws_cgi_remove_and_kill(wsi->parent);
+			/*
+			 * We need to keep the logical cgi around so we can
+			 * drain it
+			 */
+
+//			if (wsi->parent->child_list == wsi && !wsi->sibling_list)
+//				lws_cgi_remove_and_kill(wsi->parent);
 
 			/* end the binding between us and master */
-			if (wsi->parent->http.cgi)
+			if (wsi->parent->http.cgi && wsi->parent->http.cgi->lsp)
 				wsi->parent->http.cgi->lsp->stdwsi[(int)wsi->lsp_channel] =
 									NULL;
 		}
@@ -385,9 +415,9 @@ __lws_close_free_wsi(struct lws *wsi, enum lws_close_status reason,
 	    lwsi_state(wsi) == LRS_H1C_ISSUE_HANDSHAKE)
 		goto just_kill_connection;
 
-	if (!wsi->told_user_closed && wsi->user_space && wsi->protocol &&
+	if (!wsi->told_user_closed && wsi->user_space && wsi->a.protocol &&
 	    wsi->protocol_bind_balance) {
-		wsi->protocol->callback(wsi,
+		wsi->a.protocol->callback(wsi,
 				wsi->role_ops->protocol_unbind_cb[
 				       !!lwsi_role_server(wsi)],
 				       wsi->user_space, (void *)__func__, 0);
@@ -427,8 +457,13 @@ just_kill_connection:
 		lws_buflist_destroy_all_segments(&wsi->http.buflist_post_body);
 #endif
 #if defined(LWS_WITH_UDP)
-	if (wsi->udp)
+	if (wsi->udp) {
+		/* confirm no sul left scheduled in wsi->udp itself */
+		lws_sul_debug_zombies(wsi->a.context, wsi->udp,
+					sizeof(*wsi->udp), "close udp wsi");
+
 		lws_free_set_NULL(wsi->udp);
+	}
 #endif
 
 	if (wsi->role_ops->close_kill_connection)
@@ -437,11 +472,11 @@ just_kill_connection:
 	n = 0;
 
 	if (!wsi->told_user_closed && wsi->user_space &&
-	    wsi->protocol_bind_balance && wsi->protocol) {
+	    wsi->protocol_bind_balance && wsi->a.protocol) {
 		lwsl_debug("%s: %p: DROP_PROTOCOL %s\n", __func__, wsi,
-			   wsi->protocol ? wsi->protocol->name: "NULL");
-		if (wsi->protocol)
-			wsi->protocol->callback(wsi,
+			   wsi->a.protocol ? wsi->a.protocol->name: "NULL");
+		if (wsi->a.protocol)
+			wsi->a.protocol->callback(wsi,
 				wsi->role_ops->protocol_unbind_cb[
 				       !!lwsi_role_server(wsi)],
 				       wsi->user_space, (void *)__func__, 0);
@@ -452,9 +487,13 @@ just_kill_connection:
 	if ((lwsi_state(wsi) == LRS_WAITING_SERVER_REPLY ||
 	     lwsi_state(wsi) == LRS_WAITING_DNS ||
 	     lwsi_state(wsi) == LRS_WAITING_CONNECT) &&
-	     !wsi->already_did_cce && wsi->protocol) {
+	     !wsi->already_did_cce && wsi->a.protocol) {
 		static const char _reason[] = "closed before established";
 
+		lwsl_debug("%s: closing in unestablished state 0x%x\n",
+				__func__, lwsi_state(wsi));
+		wsi->socket_is_permanently_unusable = 1;
+
 		lws_inform_client_conn_fail(wsi,
 			(void *)_reason, sizeof(_reason));
 	}
@@ -546,10 +585,7 @@ just_kill_connection:
 	// lwsl_notice("%s: wsi %p, fd %d\n", __func__, wsi, wsi->desc.sockfd);
 
 	/* checking return redundant since we anyway close */
-	if (wsi->desc.sockfd != LWS_SOCK_INVALID)
-		__remove_wsi_socket_from_fds(wsi);
-	else
-		__lws_same_vh_protocol_remove(wsi);
+	__remove_wsi_socket_from_fds(wsi);
 
 	lwsi_set_state(wsi, LRS_DEAD_SOCKET);
 	lws_buflist_destroy_all_segments(&wsi->buflist);
@@ -587,7 +623,7 @@ just_kill_connection:
 		 */
 		ccb = 1;
 
-	pro = wsi->protocol;
+	pro = wsi->a.protocol;
 
 #if defined(LWS_WITH_CLIENT)
 	if (!ccb && (lwsi_state_PRE_CLOSE(wsi) & LWSIFS_NOT_EST) &&
@@ -597,8 +633,8 @@ just_kill_connection:
 #endif
 	if (ccb) {
 
-		if (!wsi->protocol && wsi->vhost && wsi->vhost->protocols)
-			pro = &wsi->vhost->protocols[0];
+		if (!wsi->a.protocol && wsi->a.vhost && wsi->a.vhost->protocols)
+			pro = &wsi->a.vhost->protocols[0];
 
 		if (pro)
 			pro->callback(wsi,
@@ -613,8 +649,8 @@ async_close:
 	lws_remove_child_from_any_parent(wsi);
 	wsi->socket_is_permanently_unusable = 1;
 
-	if (wsi->context->event_loop_ops->wsi_logical_close)
-		if (wsi->context->event_loop_ops->wsi_logical_close(wsi))
+	if (wsi->a.context->event_loop_ops->wsi_logical_close)
+		if (wsi->a.context->event_loop_ops->wsi_logical_close(wsi))
 			return;
 
 	__lws_close_free_wsi_final(wsi);
@@ -632,21 +668,34 @@ __lws_close_free_wsi_final(struct lws *wsi)
 		if (n)
 			lwsl_debug("closing: close ret %d\n", LWS_ERRNO);
 
+		__remove_wsi_socket_from_fds(wsi);
+		if (lws_socket_is_valid(wsi->desc.sockfd))
+			delete_from_fd(wsi->a.context, wsi->desc.sockfd);
+
+#if !defined(LWS_PLAT_FREERTOS) && !defined(WIN32) && !defined(LWS_PLAT_OPTEE)
+		delete_from_fdwsi(wsi->a.context, wsi);
+#endif
+
+		sanity_assert_no_sockfd_traces(wsi->a.context, wsi->desc.sockfd);
+
 		wsi->desc.sockfd = LWS_SOCK_INVALID;
 	}
 
 	/* outermost destroy notification for wsi (user_space still intact) */
-	if (wsi->vhost)
-		wsi->vhost->protocols[0].callback(wsi, LWS_CALLBACK_WSI_DESTROY,
+	if (wsi->a.vhost)
+		wsi->a.vhost->protocols[0].callback(wsi, LWS_CALLBACK_WSI_DESTROY,
 						  wsi->user_space, NULL, 0);
 
 #ifdef LWS_WITH_CGI
 	if (wsi->http.cgi) {
 		lws_spawn_piped_destroy(&wsi->http.cgi->lsp);
+		lws_sul_cancel(&wsi->http.cgi->sul_grace);
 		lws_free_set_NULL(wsi->http.cgi);
 	}
 #endif
 
+	__lws_wsi_remove_from_sul(wsi);
+	sanity_assert_no_wsi_traces(wsi->a.context, wsi);
 	__lws_free_wsi(wsi);
 }
 
@@ -654,7 +703,7 @@ __lws_close_free_wsi_final(struct lws *wsi)
 void
 lws_close_free_wsi(struct lws *wsi, enum lws_close_status reason, const char *caller)
 {
-	struct lws_context_per_thread *pt = &wsi->context->pt[(int)wsi->tsi];
+	struct lws_context_per_thread *pt = &wsi->a.context->pt[(int)wsi->tsi];
 
 	lws_pt_lock(pt, __func__);
 	__lws_close_free_wsi(wsi, reason, caller);
