@@ -32,9 +32,9 @@ int
 _lws_plat_service_forced_tsi(struct lws_context *context, int tsi)
 {
 	struct lws_context_per_thread *pt = &context->pt[tsi];
-	int m, n;
+	int m, n, r;
 
-	lws_service_flag_pending(context, tsi);
+	r = lws_service_flag_pending(context, tsi);
 
 	/* any socket with events to service? */
 	for (n = 0; n < (int)pt->fds_count; n++) {
@@ -51,9 +51,10 @@ _lws_plat_service_forced_tsi(struct lws_context *context, int tsi)
 
 	lws_service_do_ripe_rxflow(pt);
 
-	return 0;
+	return r;
 }
 
+extern void lws_client_conn_wait_timeout(lws_sorted_usec_list_t *sul);
 
 int
 _lws_plat_service_tsi(struct lws_context *context, int timeout_ms, int tsi)
@@ -70,19 +71,19 @@ _lws_plat_service_tsi(struct lws_context *context, int timeout_ms, int tsi)
 	int interrupt_requested;
 
 	/* stay dead once we are dead */
-	if (context == NULL || !context->vhost_list)
+	if (context == NULL)
 		return 1;
 
 	pt = &context->pt[tsi];
 
-	if (!pt->service_tid_detected) {
-		struct lws _lws;
+	if (!pt->service_tid_detected && context->vhost_list) {
+		lws_fakewsi_def_plwsa(pt);
 
-		memset(&_lws, 0, sizeof(_lws));
-		_lws.context = context;
+		lws_fakewsi_prep_plwsa_ctx(context);
 
 		pt->service_tid = context->vhost_list->
-			protocols[0].callback(&_lws, LWS_CALLBACK_GET_THREAD_ID,
+			protocols[0].callback((struct lws *)plwsa,
+					LWS_CALLBACK_GET_THREAD_ID,
 						  NULL, NULL, 0);
 		pt->service_tid_detected = 1;
 	}
@@ -125,26 +126,31 @@ _lws_plat_service_tsi(struct lws_context *context, int timeout_ms, int tsi)
 	}
 
 	/*
-	 * is there anybody with pending stuff that needs service forcing?
-	 */
-	if (!lws_service_adjust_timeout(context, 1, tsi))
-		_lws_plat_service_forced_tsi(context, tsi);
-
-	/*
-	 * service pending callbakcs and get maximum wait time
+	 * service pending callbacks and get maximum wait time
 	 */
 	{
 		lws_usec_t us;
 
 		lws_pt_lock(pt, __func__);
 		/* don't stay in poll wait longer than next hr timeout */
-		us = __lws_sul_service_ripe(&pt->pt_sul_owner, lws_now_usecs());
+		us = __lws_sul_service_ripe(pt->pt_sul_owner,
+					    LWS_COUNT_PT_SUL_OWNERS,
+					    lws_now_usecs());
 		if (us && us < timeout_us)
 			timeout_us = us;
 
 		lws_pt_unlock(pt);
 	}
 
+	if (_lws_plat_service_forced_tsi(context, tsi))
+		timeout_us = 0;
+
+	/*
+	 * is there anybody with pending stuff that needs service forcing?
+	 */
+	if (!lws_service_adjust_timeout(context, 1, tsi))
+		timeout_us = 0;
+
 	for (n = 0; n < (int)pt->fds_count; n++)
 		WSAEventSelect(pt->fds[n].fd, pt->events,
 		       FD_READ | (!!(pt->fds[n].events & LWS_POLLOUT) * FD_WRITE) |
@@ -155,70 +161,94 @@ _lws_plat_service_tsi(struct lws_context *context, int timeout_ms, int tsi)
 
 	ev = WSAWaitForMultipleEvents(1, &pt->events, FALSE,
 				      (DWORD)(timeout_us / LWS_US_PER_MS), FALSE);
-	if (ev == WSA_WAIT_EVENT_0) {
-		EnterCriticalSection(&pt->interrupt_lock);
-		interrupt_requested = pt->interrupt_requested;
-		pt->interrupt_requested = 0;
-		LeaveCriticalSection(&pt->interrupt_lock);
-		if (interrupt_requested) {
-			lws_broadcast(pt, LWS_CALLBACK_EVENT_WAIT_CANCELLED,
-				      NULL, 0);
-			return 0;
-		}
-
-#if defined(LWS_WITH_TLS)
-		if (pt->context->tls_ops &&
-		    pt->context->tls_ops->fake_POLLIN_for_buffered)
-			pt->context->tls_ops->fake_POLLIN_for_buffered(pt);
-#endif
-
-		for (eIdx = 0; eIdx < pt->fds_count; ++eIdx) {
-			unsigned int err;
-
-			if (WSAEnumNetworkEvents(pt->fds[eIdx].fd, pt->events,
-					&networkevents) == SOCKET_ERROR) {
-				lwsl_err("WSAEnumNetworkEvents() failed "
-					 "with error %d\n", LWS_ERRNO);
-				return -1;
-			}
-
-			if (!networkevents.lNetworkEvents)
-				networkevents.lNetworkEvents = LWS_POLLOUT;
-
-			pfd = &pt->fds[eIdx];
-			pfd->revents = (short)networkevents.lNetworkEvents;
-
-			err = networkevents.iErrorCode[FD_CONNECT_BIT];
-
-			if ((networkevents.lNetworkEvents & FD_CONNECT) &&
-			     err && err != LWS_EALREADY &&
-			     err != LWS_EINPROGRESS && err != LWS_EWOULDBLOCK &&
-			     err != WSAEINVAL) {
-				lwsl_debug("Unable to connect errno=%d\n", err);
-				pfd->revents |= LWS_POLLHUP;
-			}
-
-			if (pfd->revents & LWS_POLLOUT) {
-				wsi = wsi_from_fd(context, pfd->fd);
-				if (wsi)
-					wsi->sock_send_blocking = 0;
-			}
-			 /* if something closed, retry this slot */
-			if (pfd->revents & LWS_POLLHUP)
-				--eIdx;
-
-			if (pfd->revents) {
-				recv(pfd->fd, NULL, 0, 0);
-				lws_service_fd_tsi(context, pfd, tsi);
-			}
-		}
+	if (ev != WSA_WAIT_EVENT_0)
+		return 0;
 
+	EnterCriticalSection(&pt->interrupt_lock);
+	interrupt_requested = pt->interrupt_requested;
+	pt->interrupt_requested = 0;
+	LeaveCriticalSection(&pt->interrupt_lock);
+	if (interrupt_requested) {
+		lws_broadcast(pt, LWS_CALLBACK_EVENT_WAIT_CANCELLED,
+			      NULL, 0);
 		return 0;
 	}
 
-	// if (ev == WSA_WAIT_TIMEOUT) { }
-	// if (ev == WSA_WAIT_FAILED)
-		// return 0;
+#if defined(LWS_WITH_TLS)
+	if (pt->context->tls_ops &&
+	    pt->context->tls_ops->fake_POLLIN_for_buffered)
+		pt->context->tls_ops->fake_POLLIN_for_buffered(pt);
+#endif
+
+	for (eIdx = 0; eIdx < pt->fds_count; ++eIdx) {
+		unsigned int err;
+
+		if (WSAEnumNetworkEvents(pt->fds[eIdx].fd, pt->events,
+				&networkevents) == SOCKET_ERROR) {
+			lwsl_err("WSAEnumNetworkEvents() failed "
+				 "with error %d\n", LWS_ERRNO);
+			return -1;
+		}
+
+		if (!networkevents.lNetworkEvents)
+			networkevents.lNetworkEvents = LWS_POLLOUT;
+
+		pfd = &pt->fds[eIdx];
+		pfd->revents = (short)networkevents.lNetworkEvents;
+
+		err = networkevents.iErrorCode[FD_CONNECT_BIT];
+               if ((networkevents.lNetworkEvents & FD_CONNECT) && wsi_from_fd(context, pfd->fd) && !wsi_from_fd(context, pfd->fd)->udp) {
+                       lwsl_debug("%s: FD_CONNECT: %p\n", __func__, wsi_from_fd(context, pfd->fd));
+			pfd->revents &= ~LWS_POLLOUT;
+			if (err && err != LWS_EALREADY &&
+			    err != LWS_EINPROGRESS && err != LWS_EWOULDBLOCK &&
+			    err != WSAEINVAL) {
+				lwsl_debug("Unable to connect errno=%d\n", err);
+
+				/*
+				 * the connection has definitively failed... but
+				 * do we have more DNS entries to try?
+				 */
+				if (wsi_from_fd(context, pfd->fd)->dns_results_next) {
+					lws_sul_schedule(context, 0, &wsi_from_fd(context, pfd->fd)->
+									sul_connect_timeout,
+							 lws_client_conn_wait_timeout, 1);
+					continue;
+                               } else
+					pfd->revents |= LWS_POLLHUP;
+			} else
+                               if (wsi_from_fd(context, pfd->fd)) {
+                                       if (wsi_from_fd(context, pfd->fd)->udp)
+                                               pfd->revents |= LWS_POLLHUP;
+                                       else
+                                               lws_client_connect_3_connect(wsi_from_fd(context, pfd->fd),
+								NULL, NULL, LWS_CONNECT_COMPLETION_GOOD,
+								NULL);
+                               }
+		}
+
+		if (pfd->revents & LWS_POLLOUT) {
+			wsi = wsi_from_fd(context, pfd->fd);
+			if (wsi)
+				wsi->sock_send_blocking = 0;
+		}
+		 /* if something closed, retry this slot */
+		if (pfd->revents & LWS_POLLHUP)
+			--eIdx;
+
+		if (pfd->revents) {
+			/*
+			 * On windows is somehow necessary to "acknowledge" the
+			 * POLLIN event, otherwise we never receive another one
+			 * on the TCP connection.  But it breaks UDP, so only
+			 * do it on non-UDP.
+			 */
+			if (!wsi_from_fd(context, pfd->fd)->udp)
+				recv(pfd->fd, NULL, 0, 0);
+
+			lws_service_fd_tsi(context, pfd, tsi);
+		}
+	}
 
 	return 0;
 }
