@@ -214,8 +214,8 @@ lws_async_dns_writeable(struct lws *wsi, lws_adns_q_t *q)
 	n = lws_ptr_diff(p, pkt + LWS_PRE);
 	m = lws_write(wsi, pkt + LWS_PRE, n, 0);
 	if (m != n) {
-		lwsl_notice("%s: dns write failed %d %d\n", __func__,
-			    m, n);
+		lwsl_notice("%s: dns write failed %d %d errno %d\n", __func__,
+			    m, n, errno);
 		goto qfail;
 	}
 
@@ -298,6 +298,11 @@ lws_async_dns_init(struct lws_context *context)
 	char ads[48];
 	int n;
 
+	if (!context->vhost_list) { /* coverity... system vhost always present */
+		lwsl_err("%s: no system vhost\n", __func__);
+		return 1;
+	}
+
 	memset(&dns->sa46, 0, sizeof(dns->sa46));
 
 #if defined(LWS_WITH_SYS_DHCP_CLIENT)
@@ -472,7 +477,7 @@ cancel(struct lws_dll2 *d, void *user)
 void
 lws_async_dns_cancel(struct lws *wsi)
 {
-	lws_async_dns_t *dns = &wsi->context->async_dns;
+	lws_async_dns_t *dns = &wsi->a.context->async_dns;
 
 	lws_dll2_foreach_safe(&dns->waiting, wsi, cancel);
 }
